package com.example.coroutinesbug

import android.app.Activity
import android.os.Bundle
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.SupervisorJob
import kotlinx.coroutines.cancel
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.flow.flatMapLatest
import kotlinx.coroutines.flow.flowOf
import kotlinx.coroutines.flow.onEach
import kotlinx.coroutines.isActive
import kotlinx.coroutines.launch

class ExampleActivity : Activity() {
    @OptIn(ExperimentalCoroutinesApi::class)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val scope = CoroutineScope(SupervisorJob() + Dispatchers.Main.immediate)
        println("Before launch")
        scope.launch {
            flowOf(listOf(1, 2, 3))
                .flatMapLatest { numbers ->
                    combine(numbers.map { flowOf(it) }) { it.sum() }
                }
                .onEach { println("Got item") }
                .first()
            println("After first, scope is active: ${scope.isActive}")
        }
        println("After launch")
        scope.cancel()
    }
}